"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const BaseRoute_1 = require("./BaseRoute");
const tsioc_1 = require("tsioc");
const decorators_1 = require("../decorators");
const decorators_2 = require("../decorators");
const util_1 = require("../util");
const index_1 = require("../errors/index");
const util_2 = require("util");
class ControllerRoute extends BaseRoute_1.BaseRoute {
    constructor(route, controller) {
        super(route);
        this.controller = controller;
    }
    async navigating(container, ctx) {
        let routPath = ctx.url.replace(this.url, '');
        let respone;
        try {
            switch (ctx.method) {
                case 'GET':
                    respone = await this.invoke(container, decorators_1.Get, routPath, (meta, params, ctrl) => this.createProvider(container, meta, params, ctrl, ctx));
                    break;
                case 'POST':
                    respone = await this.invoke(container, decorators_1.Post, routPath, (meta, params, ctrl) => this.createProvider(container, meta, params, ctrl, ctx));
                    break;
                case 'Put':
                    respone = await this.invoke(container, decorators_1.Put, routPath, (meta, params, ctrl) => this.createProvider(container, meta, params, ctrl, ctx));
                    break;
                case 'DElETE':
                    respone = await this.invoke(container, decorators_1.Delete, routPath, (meta, params, ctrl) => this.createProvider(container, meta, params, ctrl, ctx));
                    break;
                default:
                    break;
            }
            ctx.body = respone;
        }
        catch (err) {
            if (err instanceof index_1.HttpError) {
                ctx.status = err.code;
                ctx.response.status = err.code;
                ctx.response.message = err.message;
            }
            console.log(err.toString());
        }
    }
    createProvider(container, meta, params, ctrl, ctx) {
        if (params && params.length) {
            if (this.isRestUri(meta.route)) {
                let url = meta.route.substr(0, meta.route.indexOf('/:')) + '/';
                let querystring = ctx.url.replace(this.url + url, '');
                let paramVal = querystring.indexOf('/') > 0 ? querystring.substr(0, querystring.indexOf('/')) : querystring;
                let body = ctx.request['body'] || {};
                let providers = params.map((p, idx) => {
                    try {
                        if (!this.isBaseType(p)) {
                            let val = container.get(p);
                            for (let n in val) {
                                if (!util_2.isUndefined(body[n])) {
                                    val[n] = body[n];
                                }
                            }
                            return {
                                value: val,
                                index: idx
                            };
                        }
                        else {
                            let val;
                            if (paramVal !== null) {
                                if (p === String) {
                                    val = paramVal;
                                }
                                else if (p === Boolean) {
                                    val = new Boolean(paramVal);
                                }
                                else if (p === Number) {
                                    val = parseFloat(paramVal);
                                }
                                else if (p === Date) {
                                    val = new Date(paramVal);
                                }
                                paramVal = null;
                            }
                            return {
                                value: val,
                                index: idx
                            };
                        }
                    }
                    catch (err) {
                        throw new index_1.BadRequestError(err.toString());
                    }
                });
                return providers;
            }
        }
        return [];
    }
    isBaseType(p) {
        if (!tsioc_1.isToken(p)) {
            return true;
        }
        if (p === Boolean || p === String || p === Number) {
            return true;
        }
        if (!tsioc_1.isFunction(p)) {
            return true;
        }
        return false;
    }
    isRestUri(uri) {
        return /\/:/.test(uri || '');
    }
    async invoke(container, decorator, routPath, provider) {
        let decoratorName = decorator.toString();
        let methodMaps = tsioc_1.getMethodMetadata(decoratorName, this.controller);
        let meta;
        let allMethods = [];
        for (let name in methodMaps) {
            allMethods = allMethods.concat(methodMaps[name]);
        }
        allMethods = allMethods.sort((ra, rb) => (rb.route || '').length - (ra.route || '').length);
        meta = allMethods.find(route => {
            let uri = route.route || '';
            if (uri === routPath) {
                return true;
            }
            if (this.isRestUri(uri)) {
                let idex = uri.indexOf('/:');
                let url = uri.substr(0, idex);
                if (url !== routPath && routPath.indexOf(url) === 0) {
                    return true;
                }
            }
            return false;
        });
        if (meta && meta.propertyKey) {
            let ctrl = container.get(this.controller);
            if (container.has(util_1.symbols.IAuthorization)) {
                let hasAuth = Reflect.hasMetadata(decorators_2.Authorization.toString(), ctrl, meta.propertyKey);
                if (hasAuth) {
                    let auth = container.get(util_1.symbols.IAuthorization);
                    if (!auth.isAuth()) {
                        throw new index_1.UnauthorizedError();
                    }
                }
            }
            let params = container.getMethodParameters(this.controller, ctrl, meta.propertyKey);
            return container.invoke(this.controller, meta.propertyKey, ctrl, ...provider(meta, params, ctrl));
        }
        else {
            throw new index_1.NotFoundError();
        }
    }
}
exports.ControllerRoute = ControllerRoute;

//# sourceMappingURL=../sourcemaps/router/ControllerRoute.js.map
